defmodule Tradex.Instruments do
  @moduledoc """
  The Instruments context.
  """

  import Ecto.Query, warn: false

  alias Tradex.Instruments.Security
  alias Tradex.Repo

  @doc """
  Returns the list of securities.

  ## Examples

      iex> list_securities()
      [%Security{}, ...]

  """
  def list_securities do
    Repo.all(Security)
  end

  @doc """
  Gets a single security.

  Raises `Ecto.NoResultsError` if the Security does not exist.

  ## Examples

      iex> get_security!(123)
      %Security{}

      iex> get_security!(456)
      ** (Ecto.NoResultsError)

  """
  def get_security!(id), do: Repo.get!(Security, id)

  @doc """
  Creates a security.

  ## Examples

      iex> create_security(%{field: value})
      {:ok, %Security{}}

      iex> create_security(%{field: bad_value})
      {:error, %Ecto.Changeset{}}

  """
  def create_security(attrs \\ %{}) do
    %Security{}
    |> Security.changeset(attrs)
    |> Repo.insert()
  end

  @doc """
  Updates a security.

  ## Examples

      iex> update_security(security, %{field: new_value})
      {:ok, %Security{}}

      iex> update_security(security, %{field: bad_value})
      {:error, %Ecto.Changeset{}}

  """
  def update_security(%Security{} = security, attrs) do
    security
    |> Security.changeset(attrs)
    |> Repo.update()
  end

  @doc """
  Deletes a security.

  ## Examples

      iex> delete_security(security)
      {:ok, %Security{}}

      iex> delete_security(security)
      {:error, %Ecto.Changeset{}}

  """
  def delete_security(%Security{} = security) do
    Repo.delete(security)
  end

  @doc """
  Returns an `%Ecto.Changeset{}` for tracking security changes.

  ## Examples

      iex> change_security(security)
      %Ecto.Changeset{data: %Security{}}

  """
  def change_security(%Security{} = security, attrs \\ %{}) do
    Security.changeset(security, attrs)
  end
end
